@@ -92,11 +92,12 @@ struct xdpw_screencast_instance {
 	uint32_t seq;
 	uint32_t node_id;
 	bool pwr_stream_state;
+	uint32_t framerate;
 
 	// wlroots
 	struct zwlr_screencopy_frame_v1 *frame_callback;
 	struct xdpw_wlr_output *target_output;
-	uint32_t framerate;
+	uint32_t max_framerate;
 	struct zwlr_screencopy_frame_v1 *wlr_frame;
 	struct xdpw_frame simple_frame;
 	bool with_cursor;
@@ -121,8 +122,8 @@ struct xdpw_wlr_output {
 };
 
 void randname(char *buf);
-enum spa_video_format xdpw_format_pw_from_wl_shm(
-	struct xdpw_screencast_instance *cast);
+int anonymous_shm_open(void);
+enum spa_video_format xdpw_format_pw_from_wl_shm(enum wl_shm_format format);
 enum spa_video_format xdpw_format_pw_strip_alpha(enum spa_video_format format);
 
 enum xdpw_chooser_types get_chooser_type(const char *chooser_type);
