@@ -5,6 +5,8 @@
 #include <spa/param/props.h>
 #include <spa/param/format-utils.h>
 #include <spa/param/video/format-utils.h>
+#include <sys/mman.h>
+#include <unistd.h>
 
 #include "wlr_screencast.h"
 #include "xdpw.h"
@@ -26,6 +28,37 @@ static void writeFrameData(void *pwFramePointer, void *wlrFramePointer,
 	return;
 }
 
+static struct spa_pod *build_format(struct spa_pod_builder *b, enum spa_video_format format,
+		uint32_t width, uint32_t height, uint32_t framerate) {
+	struct spa_pod_frame f[1];
+
+	enum spa_video_format format_without_alpha = xdpw_format_pw_strip_alpha(format);
+
+	spa_pod_builder_push_object(b, &f[0], SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat);
+	spa_pod_builder_add(b, SPA_FORMAT_mediaType, SPA_POD_Id(SPA_MEDIA_TYPE_video), 0);
+	spa_pod_builder_add(b, SPA_FORMAT_mediaSubtype, SPA_POD_Id(SPA_MEDIA_SUBTYPE_raw), 0);
+	/* format */
+	if (format_without_alpha == SPA_VIDEO_FORMAT_UNKNOWN) {
+		spa_pod_builder_add(b, SPA_FORMAT_VIDEO_format, SPA_POD_Id(format), 0);
+	} else {
+		spa_pod_builder_add(b, SPA_FORMAT_VIDEO_format,
+				SPA_POD_CHOICE_ENUM_Id(3, format, format, format_without_alpha), 0);
+	}
+	spa_pod_builder_add(b, SPA_FORMAT_VIDEO_size,
+		SPA_POD_Rectangle(&SPA_RECTANGLE(width, height)),
+		0);
+	// variable framerate
+	spa_pod_builder_add(b, SPA_FORMAT_VIDEO_framerate,
+		SPA_POD_Fraction(&SPA_FRACTION(0, 1)), 0);
+	spa_pod_builder_add(b, SPA_FORMAT_VIDEO_maxFramerate,
+		SPA_POD_CHOICE_RANGE_Fraction(
+			&SPA_FRACTION(framerate, 1),
+			&SPA_FRACTION(1, 1),
+			&SPA_FRACTION(framerate, 1)),
+		0);
+	return spa_pod_builder_pop(b, &f[0]);
+}
+
 static void pwr_on_event(void *data, uint64_t expirations) {
 	struct xdpw_screencast_instance *cast = data;
 	struct pw_buffer *pw_buf;
@@ -54,15 +87,6 @@ static void pwr_on_event(void *data, uint64_t expirations) {
 		h->dts_offset = 0;
 	}
 
-	d[0].type = SPA_DATA_MemPtr;
-	d[0].maxsize = cast->simple_frame.size;
-	d[0].mapoffset = 0;
-	d[0].chunk->size = cast->simple_frame.size;
-	d[0].chunk->stride = cast->simple_frame.stride;
-	d[0].chunk->offset = 0;
-	d[0].flags = 0;
-	d[0].fd = -1;
-
 	writeFrameData(d[0].data, cast->simple_frame.data, cast->simple_frame.height,
 		cast->simple_frame.stride, cast->simple_frame.y_invert);
 
@@ -87,7 +111,7 @@ static void pwr_handle_stream_state_changed(void *data,
 
 	logprint(INFO, "pipewire: stream state changed to \"%s\"",
 		pw_stream_state_as_string(state));
-	logprint(INFO, "pipewire: node id is %d", cast->node_id);
+	logprint(INFO, "pipewire: node id is %d", (int)cast->node_id);
 
 	switch (state) {
 	case PW_STREAM_STATE_STREAMING:
@@ -101,6 +125,7 @@ static void pwr_handle_stream_state_changed(void *data,
 
 static void pwr_handle_stream_param_changed(void *data, uint32_t id,
 		const struct spa_pod *param) {
+	logprint(TRACE, "pipewire: stream parameters changed");
 	struct xdpw_screencast_instance *cast = data;
 	struct pw_stream *stream = cast->stream;
 	uint8_t params_buffer[1024];
@@ -113,6 +138,7 @@ static void pwr_handle_stream_param_changed(void *data, uint32_t id,
 	}
 
 	spa_format_video_raw_parse(param, &cast->pwr_format);
+	cast->framerate = (uint32_t)(cast->pwr_format.max_framerate.num / cast->pwr_format.max_framerate.denom);
 
 	params[0] = spa_pod_builder_add_object(&b,
 		SPA_TYPE_OBJECT_ParamBuffers, SPA_PARAM_Buffers,
@@ -120,7 +146,8 @@ static void pwr_handle_stream_param_changed(void *data, uint32_t id,
 		SPA_PARAM_BUFFERS_blocks,  SPA_POD_Int(1),
 		SPA_PARAM_BUFFERS_size,    SPA_POD_Int(cast->simple_frame.size),
 		SPA_PARAM_BUFFERS_stride,  SPA_POD_Int(cast->simple_frame.stride),
-		SPA_PARAM_BUFFERS_align,   SPA_POD_Int(XDPW_PWR_ALIGN));
+		SPA_PARAM_BUFFERS_align,   SPA_POD_Int(XDPW_PWR_ALIGN),
+		SPA_PARAM_BUFFERS_dataType,SPA_POD_CHOICE_FLAGS_Int(1<<SPA_DATA_MemFd));
 
 	params[1] = spa_pod_builder_add_object(&b,
 		SPA_TYPE_OBJECT_ParamMeta, SPA_PARAM_Meta,
@@ -130,12 +157,96 @@ static void pwr_handle_stream_param_changed(void *data, uint32_t id,
 	pw_stream_update_params(stream, params, 2);
 }
 
+static void pwr_handle_stream_add_buffer(void *data, struct pw_buffer *buffer) {
+	struct xdpw_screencast_instance *cast = data;
+	struct spa_data *d;
+
+	logprint(TRACE, "pipewire: add buffer event handle");
+
+	d = buffer->buffer->datas;
+
+	// Select buffer type from negotiation result
+	if ((d[0].type & (1u << SPA_DATA_MemFd)) > 0) {
+		d[0].type = SPA_DATA_MemFd;
+	} else {
+		logprint(ERROR, "pipewire: unsupported buffer type");
+		cast->err = 1;
+		return;
+	}
+
+	logprint(TRACE, "pipewire: selected buffertype %u", d[0].type);
+	// Prepare buffer for choosen type
+	if (d[0].type == SPA_DATA_MemFd) {
+		d[0].maxsize = cast->simple_frame.size;
+		d[0].mapoffset = 0;
+		d[0].chunk->size = cast->simple_frame.size;
+		d[0].chunk->stride = cast->simple_frame.stride;
+		d[0].chunk->offset = 0;
+		d[0].flags = 0;
+		d[0].fd = anonymous_shm_open();
+
+		if (d[0].fd == -1) {
+			logprint(ERROR, "pipewire: unable to create anonymous filedescriptor");
+			cast->err = 1;
+			return;
+		}
+
+		if (ftruncate(d[0].fd, d[0].maxsize) < 0) {
+			logprint(ERROR, "pipewire: unable to truncate filedescriptor");
+			close(d[0].fd);
+			d[0].fd = -1;
+			cast->err = 1;
+			return;
+		}
+
+		// mmap buffer, so we can use the data_ptr in on_process
+		d[0].data = mmap(NULL, d[0].maxsize, PROT_READ | PROT_WRITE, MAP_SHARED, d[0].fd, d[0].mapoffset);
+		if (d[0].data == MAP_FAILED) {
+			logprint(ERROR, "pipewire: unable to mmap memory");
+			cast->err = 1;
+			return;
+		}
+	}
+}
+
+static void pwr_handle_stream_remove_buffer(void *data, struct pw_buffer *buffer) {
+	logprint(TRACE, "pipewire: remove buffer event handle");
+
+	struct spa_data *d = buffer->buffer->datas;
+	switch (d[0].type) {
+	case SPA_DATA_MemFd:
+		munmap(d[0].data, d[0].maxsize);
+		close(d[0].fd);
+		break;
+	default:
+		break;
+	}
+}
+
 static const struct pw_stream_events pwr_stream_events = {
 	PW_VERSION_STREAM_EVENTS,
 	.state_changed = pwr_handle_stream_state_changed,
 	.param_changed = pwr_handle_stream_param_changed,
+	.add_buffer = pwr_handle_stream_add_buffer,
+	.remove_buffer = pwr_handle_stream_remove_buffer,
 };
 
+void pwr_update_stream_param(struct xdpw_screencast_instance *cast) {
+	logprint(TRACE, "pipewire: stream update parameters");
+	struct pw_stream *stream = cast->stream;
+	uint8_t params_buffer[1024];
+	struct spa_pod_builder b =
+		SPA_POD_BUILDER_INIT(params_buffer, sizeof(params_buffer));
+	const struct spa_pod *params[1];
+
+	enum spa_video_format format = xdpw_format_pw_from_wl_shm(cast->simple_frame.format);
+
+	params[0] = build_format(&b, format,
+			cast->simple_frame.width, cast->simple_frame.height, cast->framerate);
+
+	pw_stream_update_params(stream, params, 1);
+}
+
 void xdpw_pwr_stream_create(struct xdpw_screencast_instance *cast) {
 	struct xdpw_screencast_context *ctx = cast->ctx;
 	struct xdpw_state *state = ctx->state;
@@ -163,37 +274,10 @@ void xdpw_pwr_stream_create(struct xdpw_screencast_instance *cast) {
 		pw_loop_add_event(state->pw_loop, pwr_on_event, cast);
 	logprint(DEBUG, "pipewire: registered event %p", cast->event);
 
-	enum spa_video_format format = xdpw_format_pw_from_wl_shm(cast);
-	enum spa_video_format format_without_alpha =
-		xdpw_format_pw_strip_alpha(format);
+	enum spa_video_format format = xdpw_format_pw_from_wl_shm(cast->simple_frame.format);
 
-	struct spa_pod_frame f;
-	spa_pod_builder_push_object(&b, &f, SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat);
-	spa_pod_builder_add(&b, SPA_FORMAT_mediaType, SPA_POD_Id(SPA_MEDIA_TYPE_video), 0);
-	spa_pod_builder_add(&b, SPA_FORMAT_mediaSubtype, SPA_POD_Id(SPA_MEDIA_SUBTYPE_raw), 0);
-	if (format_without_alpha != SPA_VIDEO_FORMAT_UNKNOWN) {
-		spa_pod_builder_add(&b, SPA_FORMAT_VIDEO_format,
-			SPA_POD_CHOICE_ENUM_Id(3, format, format, format_without_alpha), 0);
-	} else {
-		spa_pod_builder_add(&b, SPA_FORMAT_VIDEO_format,
-			SPA_POD_CHOICE_ENUM_Id(2, format, format), 0);
-	}
-	spa_pod_builder_add(&b, SPA_FORMAT_VIDEO_size,
-		SPA_POD_CHOICE_RANGE_Rectangle(
-			&SPA_RECTANGLE(cast->simple_frame.width, cast->simple_frame.height),
-			&SPA_RECTANGLE(1, 1),
-			&SPA_RECTANGLE(4096, 4096)),
-		0);
-	// variable framerate
-	spa_pod_builder_add(&b, SPA_FORMAT_VIDEO_framerate,
-		SPA_POD_Fraction(&SPA_FRACTION(0, 1)), 0);
-	spa_pod_builder_add(&b, SPA_FORMAT_VIDEO_maxFramerate,
-		SPA_POD_CHOICE_RANGE_Fraction(
-			&SPA_FRACTION(cast->framerate, 1),
-			&SPA_FRACTION(1, 1),
-			&SPA_FRACTION(cast->framerate, 1)),
-		0);
-	const struct spa_pod *param = spa_pod_builder_pop(&b, &f);
+	const struct spa_pod *param = build_format(&b, format,
+			cast->simple_frame.width, cast->simple_frame.height, cast->framerate);
 
 	pw_stream_add_listener(cast->stream, &cast->stream_listener,
 		&pwr_stream_events, cast);
@@ -202,7 +286,7 @@ void xdpw_pwr_stream_create(struct xdpw_screencast_instance *cast) {
 		PW_DIRECTION_OUTPUT,
 		PW_ID_ANY,
 		(PW_STREAM_FLAG_DRIVER |
-			PW_STREAM_FLAG_MAP_BUFFERS),
+			PW_STREAM_FLAG_ALLOC_BUFFERS),
 		&param, 1);
 }
 
