@@ -19,20 +19,30 @@ static const char object_path[] = "/org/freedesktop/portal/desktop";
 static const char interface_name[] = "org.freedesktop.impl.portal.ScreenCast";
 
 void exec_with_shell(char *command) {
-	pid_t pid = fork();
-	if (pid < 0) {
+	pid_t pid1 = fork();
+	if (pid1 < 0) {
 		perror("fork");
-	} else if (pid == 0) {
-		char *const argv[] = {
-			"sh",
-			"-c",
-			command,
-			NULL,
-		};
-		execvp("sh", argv);
-
-		perror("execvp");
-		exit(127);
+		return;
+	} else if (pid1 == 0) {
+		pid_t pid2 = fork();
+		if (pid2 < 0) {
+			perror("fork");
+		} else if (pid2 == 0) {
+			char *const argv[] = {
+				"sh",
+				"-c",
+				command,
+				NULL,
+			};
+			execvp("sh", argv);
+			perror("execvp");
+			_exit(127);
+		}
+		_exit(0);
+	}
+	int stat;
+	if (waitpid(pid1, &stat, 0) < 0) {
+		perror("waitpid");
 	}
 }
 
@@ -50,9 +60,16 @@ void xdpw_screencast_instance_init(struct xdpw_screencast_context *ctx,
 
 	cast->ctx = ctx;
 	cast->target_output = out;
-	cast->framerate = out->framerate;
+	if (ctx->state->config->screencast_conf.max_fps > 0) {
+		cast->max_framerate = ctx->state->config->screencast_conf.max_fps < (uint32_t)out->framerate ?
+			ctx->state->config->screencast_conf.max_fps : (uint32_t)out->framerate;
+	} else {
+		cast->max_framerate = (uint32_t)out->framerate;
+	}
+	cast->framerate = cast->max_framerate;
 	cast->with_cursor = with_cursor;
 	cast->refcount = 1;
+	cast->node_id = SPA_ID_INVALID;
 	logprint(INFO, "xdpw: screencast instance %p has %d references", cast, cast->refcount);
 	wl_list_insert(&ctx->screencast_instances, &cast->link);
 	logprint(INFO, "xdpw: %d active screencast instances",
@@ -261,9 +278,9 @@ static int method_screencast_select_sources(sd_bus_message *msg, void *data,
 		}
 
 		if (strcmp(key, "multiple") == 0) {
-			bool multiple;
+			int multiple;
 			sd_bus_message_read(msg, "v", "b", &multiple);
-			logprint(INFO, "dbus: option multiple: %x", multiple);
+			logprint(INFO, "dbus: option multiple: %d", multiple);
 		} else if (strcmp(key, "types") == 0) {
 			uint32_t mask;
 			sd_bus_message_read(msg, "v", "u", &mask);
@@ -413,7 +430,7 @@ static int method_screencast_start(sd_bus_message *msg, void *data,
 		start_screencast(cast);
 	}
 
-	while (cast->node_id == 0) {
+	while (cast->node_id == SPA_ID_INVALID) {
 		int ret = pw_loop_iterate(state->pw_loop, 0);
 		if (ret != 0) {
 			logprint(ERROR, "pipewire_loop_iterate failed: %s", spa_strerror(ret));
@@ -426,6 +443,7 @@ static int method_screencast_start(sd_bus_message *msg, void *data,
 		return ret;
 	}
 
+	logprint(DEBUG, "dbus: start: returning node %d", (int)cast->node_id);
 	ret = sd_bus_message_append(reply, "ua{sv}", PORTAL_RESPONSE_SUCCESS, 1,
 		"streams", "a(ua{sv})", 1,
 		cast->node_id, 2,
