@@ -52,7 +52,7 @@ void xdpw_wlr_frame_free(struct xdpw_screencast_instance *cast) {
 		return ;
 	}
 
-	uint64_t delay_ns = fps_limit_measure_end(&cast->fps_limit, cast->ctx->state->config->screencast_conf.max_fps);
+	uint64_t delay_ns = fps_limit_measure_end(&cast->fps_limit, cast->framerate);
 	if (delay_ns > 0) {
 		xdpw_add_timer(cast->ctx->state, delay_ns,
 			(xdpw_event_loop_timer_func_t) xdpw_wlr_register_cb, cast);
@@ -61,25 +61,6 @@ void xdpw_wlr_frame_free(struct xdpw_screencast_instance *cast) {
 	}
 }
 
-static int anonymous_shm_open(void) {
-	char name[] = "/xdpw-shm-XXXXXX";
-	int retries = 100;
-
-	do {
-		randname(name + strlen(name) - 6);
-
-		--retries;
-		// shm_open guarantees that O_CLOEXEC is set
-		int fd = shm_open(name, O_RDWR | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
-		if (fd >= 0) {
-			shm_unlink(name);
-			return fd;
-		}
-	} while (retries > 0 && errno == EEXIST);
-
-	return -1;
-}
-
 static struct wl_buffer *create_shm_buffer(struct xdpw_screencast_instance *cast,
 		enum wl_shm_format fmt, int width, int height, int stride,
 		void **data_out) {
@@ -127,6 +108,11 @@ static void wlr_frame_buffer_chparam(struct xdpw_screencast_instance *cast,
 	cast->simple_frame.size = stride * height;
 	cast->simple_frame.format = format;
 	wlr_frame_buffer_destroy(cast);
+
+	if (cast->pwr_stream_state) {
+		logprint(DEBUG, "wlroots: request pipewire param change");
+		pwr_update_stream_param(cast);
+	}
 }
 
 static void wlr_frame_linux_dmabuf(void *data,
@@ -144,7 +130,7 @@ static void wlr_frame_buffer_done(void *data,
 	zwlr_screencopy_frame_v1_copy_with_damage(frame, cast->simple_frame.buffer);
 	logprint(TRACE, "wlroots: frame copied");
 
-	fps_limit_measure_start(&cast->fps_limit, cast->ctx->state->config->screencast_conf.max_fps);
+	fps_limit_measure_start(&cast->fps_limit, cast->framerate);
 }
 
 static void wlr_frame_buffer(void *data, struct zwlr_screencopy_frame_v1 *frame,
